@@ -422,6 +422,10 @@ print.tlsConfig <- function(x, ...) {
 #'
 `$<-.nanoObject` <- function(x, name, value) x
 
+#' @export
+#'
+`[.recvAio` <- function(x, i) collect_aio_(x)
+
 #' @export
 #'
 `$<-.recvAio` <- function(x, name, value) x
@@ -430,6 +434,10 @@ print.tlsConfig <- function(x, ...) {
 #'
 `[[<-.recvAio` <- function(x, i, value) x
 
+#' @export
+#'
+`[.sendAio` <- function(x, i) collect_aio_(x)
+
 #' @export
 #'
 `$<-.sendAio` <- function(x, name, value) x
