@@ -34,14 +34,20 @@
 #include <nng/protocol/survey0/respond.h>
 #endif
 
-#ifdef NANONEXT_HTTP
-#include <nng/supplemental/http/http.h>
-#endif
-
 #ifdef NANONEXT_SUPPLEMENTALS
+#include <nng/supplemental/http/http.h>
 #include <nng/supplemental/tls/tls.h>
 #include <nng/supplemental/util/platform.h>
 
+typedef union nano_opt_u {
+  char *str;
+  bool b;
+  nng_duration d;
+  int i;
+  size_t s;
+  uint64_t u;
+} nano_opt;
+
 typedef struct nano_listener_s {
   nng_listener list;
   nng_tls_config *tls;
@@ -87,6 +93,14 @@ typedef struct nano_aio_s {
   void *next;
 } nano_aio;
 
+typedef struct nano_handle_s {
+  nng_url *url;
+  nng_http_client *cli;
+  nng_http_req *req;
+  nng_http_res *res;
+  nng_tls_config *cfg;
+} nano_handle;
+
 typedef struct nano_cv_s {
   int condition;
   int flag;
@@ -94,6 +108,23 @@ typedef struct nano_cv_s {
   nng_cv *cv;
 } nano_cv;
 
+typedef struct nano_cv_duo_s {
+  nano_cv *cv;
+  nano_cv *cv2;
+} nano_cv_duo;
+
+typedef struct nano_thread_aio_s {
+  nng_thread *thr;
+  nano_cv *cv;
+  nng_aio *aio;
+} nano_thread_aio;
+
+typedef struct nano_thread_duo_s {
+  nng_thread *thr;
+  nano_cv *cv;
+  nano_cv *cv2;
+} nano_thread_duo;
+
 #endif
 
 #ifdef NANONEXT_SIGNALS
@@ -103,8 +134,11 @@ typedef struct nano_cv_s {
 #include <signal.h>
 #endif
 
-#ifdef NANONEXT_TIME
+#ifdef NANONEXT_IO
 #include <time.h>
+#include <unistd.h>
+#include <stdarg.h>
+#include <stdio.h>
 #endif
 
 #ifdef NANONEXT_TLS
@@ -148,8 +182,19 @@ typedef struct nano_cv_s {
 #define STRICT_R_HEADERS
 #include <R.h>
 #include <Rinternals.h>
+#include <Rversion.h>
 #include <R_ext/Visibility.h>
 
+#define NANO_TAG(x) TAG(x)
+#define NANO_PTR(x) (void *) CAR(x)
+#define NANO_PROT(x) CDR(x)
+#define NANO_SET_TAG(x, v) SET_TAG(x, v)
+#define NANO_SET_PROT(x, v) SETCDR(x, v)
+#define NANO_DATAPTR(x) (void *) DATAPTR_RO(x)
+#define NANO_VECTOR(x) ((const SEXP *) DATAPTR_RO(x))
+#define NANO_INTEGER(x) *(int *) DATAPTR_RO(x)
+#define NANO_ERROR(x) { Rf_error(x); return R_NilValue; }
+
 #define ERROR_OUT(xc) Rf_error("%d | %s", xc, nng_strerror(xc))
 #define ERROR_RET(xc) { Rf_warning("%d | %s", xc, nng_strerror(xc)); return mk_error(xc); }
 #define NANONEXT_INIT_BUFSIZE 8192
@@ -166,8 +211,6 @@ typedef struct nano_cv_s {
   (x)->len = 0;                                                \
   (x)->cur = sz
 #define NANO_FREE(x) if (x.len) R_Free(x.buf)
-#define NANO_INTEGER(x) (int *) DATAPTR_RO(x)
-#define NANO_ERROR(x) { Rf_error(x); return R_NilValue; }
 #define NANO_CLASS2(x, cls1, cls2)                             \
   SEXP klass = Rf_allocVector(STRSXP, 2);                      \
   Rf_classgets(x, klass);                                      \
@@ -183,10 +226,10 @@ void later2(void (*)(void *), void *);
 extern void (*eln2)(void (*)(void *), void *, double, int);
 void eln2dummy(void (*)(void *), void *, double, int);
 
+int nano_integer(SEXP);
 SEXP nano_PreserveObject(SEXP);
 void nano_ReleaseObject(SEXP);
 SEXP mk_error(const int);
-SEXP mk_error_ncurl(const int);
 nano_buf nano_char_buf(const SEXP);
 SEXP nano_decode(unsigned char *, const size_t, const int);
 void nano_encode(nano_buf *, const SEXP);
@@ -211,8 +254,6 @@ SEXP rnng_aio_http_headers(SEXP);
 SEXP rnng_aio_http_status(SEXP);
 SEXP rnng_aio_result(SEXP);
 SEXP rnng_aio_stop(SEXP);
-SEXP rnng_base64dec(SEXP, SEXP);
-SEXP rnng_base64enc(SEXP, SEXP);
 SEXP rnng_clock(void);
 SEXP rnng_close(SEXP);
 SEXP rnng_ctx_close(SEXP);
@@ -264,7 +305,6 @@ SEXP rnng_status_code(SEXP);
 SEXP rnng_stream_close(SEXP);
 SEXP rnng_stream_dial(SEXP, SEXP, SEXP);
 SEXP rnng_stream_listen(SEXP, SEXP, SEXP);
-SEXP rnng_strcat(SEXP, SEXP);
 SEXP rnng_strerror(SEXP);
 SEXP rnng_subscribe(SEXP, SEXP, SEXP);
 SEXP rnng_tls_config(SEXP, SEXP, SEXP, SEXP);
@@ -284,7 +324,6 @@ extern SEXP nano_DotcallSymbol;
 extern SEXP nano_HeadersSymbol;
 extern SEXP nano_IdSymbol;
 extern SEXP nano_ListenerSymbol;
-extern SEXP nano_LNSymbol;
 extern SEXP nano_ProtocolSymbol;
 extern SEXP nano_RawSymbol;
 extern SEXP nano_ResolveSymbol;
@@ -298,17 +337,16 @@ extern SEXP nano_TlsSymbol;
 extern SEXP nano_UrlSymbol;
 extern SEXP nano_ValueSymbol;
 
-extern SEXP nano_aioFormals;
 extern SEXP nano_aioFuncMsg;
 extern SEXP nano_aioFuncRes;
 extern SEXP nano_aioNFuncs;
 extern SEXP nano_error;
 extern SEXP nano_klassString;
-extern SEXP nano_onLoad;
 extern SEXP nano_precious;
 extern SEXP nano_recvAio;
 extern SEXP nano_reqAio;
 extern SEXP nano_refHook;
+extern SEXP nano_sendAio;
 extern SEXP nano_success;
 extern SEXP nano_unresolved;
 
