@@ -11,6 +11,7 @@ nanotestp <- function(x) invisible(is.character(capture.output(print(x))) || sto
 nanotestxp <- function(x) invisible(typeof(x) == "externalptr" || stop("is not of type 'externalptr' as expected"))
 nanotesterr <- function(x, e = "")
   invisible(grepl(e, tryCatch(x, error = identity)[["message"]], fixed = TRUE) || stop("expected error message '", e, "' not generated"))
+later <- requireNamespace("later", quietly = TRUE)
 
 nng_version()
 nanotestnano(n <- nano("req", listen = "inproc://nanonext", autostart = FALSE))
@@ -77,7 +78,7 @@ nanotestz(n1$send(c("test", "", "spec"), mode = "raw", block = 500))
 nanotesti(n$recv("character", block = 500), c("test", "", "spec"))
 nanotestz(n$send(1:5, mode = "r"))
 nanotest(length(n1$recv("int", block = 500)) == 5L)
-nanotestaio(saio <- n1$send_aio(Reduce(strcat, replicate(5, random(1e3L))), mode = 3L, timeout = 900))
+nanotestaio(saio <- n1$send_aio(paste(replicate(5, random(1e3L)), collapse = ""), mode = 3L, timeout = 900))
 nanotestp(saio)
 nanotestaio(call_aio(saio))
 nanotestz(saio$result)
@@ -128,13 +129,13 @@ nanotestaio(rraio <- n1$recv_aio(mode = "int", timeout = 500))
 nanotest(is.raw(call_aio(rraio)$data))
 nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = "logical", timeout = 500))
-nanotestw(is.raw(call_aio(rraio)$data))
+nanotestw(is.raw(collect_aio(rraio)))
 nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = "numeric", timeout = 500))
-nanotest(is.raw(call_aio(rraio)$data))
+nanotest(is.raw(rraio[]))
 nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = "complex", timeout = 500))
-nanotest(is.raw(call_aio(rraio)$data))
+nanotest(is.raw(collect_aio_(rraio)))
 nanotesterr(opt(rraio[["aio"]], "false") <- 0L, "valid")
 nanotesterr(subscribe(rraio[["aio"]], "false"), "valid")
 nanotesterr(opt(rraio[["aio"]], "false"), "valid")
@@ -198,10 +199,10 @@ nanotesterr(req$opt("false", list()), "type")
 nanotestnano(ctx <- context(rep))
 nanotestp(ctx)
 nanotestaio(csaio <- req$send_aio(data.frame(), mode = "seria", timeout = 500))
-nanotest(is.integer(call_aio_(csaio)$result))
+nanotestz(call_aio_(csaio)$result)
 nanotestaio(craio <- recv_aio(ctx, timeout = 500))
-nanotest(is.list(.subset2(call_aio(craio), "data")))
-nanotest(is.integer(req$send("context test", mode ="raw", block = 500)))
+nanotest(is.list(collect_aio(craio)))
+nanotestz(req$send("context test", mode ="raw", block = 500))
 nanotest(recv(ctx, mode = "string", block = 500) == "context test")
 nanotestnn(req$send(data.frame(), mode = "seri", block = 500))
 nanotestaio(msg <- recv_aio(ctx, mode = "ser", timeout = 500))
@@ -209,9 +210,9 @@ nanotest(is.logical(.unresolved(msg)))
 nanotest(is.logical(unresolved(msg)))
 nanotest(is.data.frame(call_aio(msg)$data))
 nanotest(!unresolved(msg))
-nanotest(is.integer(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500)))
+nanotestz(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500))
 nanotestaio(msg <- recv_aio(ctx, mode = 6L, timeout = 500))
-nanotest(is.logical(call_aio(msg)$data))
+nanotest(is.logical(msg[]))
 nanotesti(collect_aio(msg), collect_aio_(msg))
 nanotestaio(err <- send_aio(ctx, msg[["data"]], mode = "next"))
 nanotestn(stop_aio(err))
@@ -223,39 +224,41 @@ nanotest(is_error_value(call_aio_(list(item = err))[["item"]][["result"]]))
 nanotest(is_error_value(collect_aio(err)))
 nanotest(is_error_value(collect_aio(list(item = err))[["item"]]))
 nanotest(is_error_value(collect_aio_(list(err))[[1L]]))
-nanotest(is.integer(req$send(serialize(NULL, NULL, ascii = TRUE), mode = 2L, block = 500)))
+nanotestz(req$send(serialize(NULL, NULL, ascii = TRUE), mode = 2L, block = 500))
 nanotestn(call_aio(recv_aio(ctx, mode = 1L, timeout = 500))[["value"]])
 nanotestaio(saio <- send_aio(ctx, as.raw(1L), mode = 2L, timeout = 500))
 nanotesti(req$recv(mode = 8L, block = 500), as.raw(1L))
 nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = 2L, recv_mode = "complex", timeout = 500))
-nanotest(is.integer(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "ra", timeout = 500)))
+nanotestz(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "ra", timeout = 500))
 nanotest(is.complex(call_aio(rek)[["data"]]))
 nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = "next", recv_mode = "serial", timeout = 500))
-nanotest(is.integer(reply(ctx, execute = identity, recv_mode = 1L, send_mode = 3L, timeout = 500)))
+nanotestz(reply(ctx, execute = identity, recv_mode = 1L, send_mode = 3L, timeout = 500))
 nanotest(is.complex(call_aio(rek)[["data"]]))
 
 nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) raw(1L), as.integer), class = "custom", vec = FALSE)))
 nanotest(length(nxt) == 2L)
 nanotest(is.function(nxt[[1L]]))
 custom <- list(`class<-`(new.env(), "custom"), new.env())
-nanotest(is.integer(req$send(custom, mode = 3L, block = 500)))
+nanotestz(req$send(custom, mode = 3L, block = 500))
 nanotest(is.integer(recv(rep, block = 500)[[1L]]))
-nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) raw(1L), as.integer), class = "custom", vec = TRUE)))
-nanotest(is.integer(send(rep, custom, mode = 3L, block = 500)))
+nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) list(x = as.raw(1L)), function(x) list(x = new.env())), class = "custom", vec = TRUE)))
+nanotestz(send(rep, custom, mode = 3L, block = 500))
+nanotest(is.list(req$recv(mode = 1L, block = 500)))
 nanotesti(next_config(nxt), nxt)
 nanotesti(next_config(as.pairlist(c("a", "b", "c"))), nxt)
 nanotest(is.list(next_config(NULL, mark = TRUE)))
 nanotestn(unlist(next_config()))
+nanotesterr(next_config(NULL, class = 1L), "must be a character string")
 
-nanotestaio(cs <- request_signal(req$context, "test", send_mode = "next", cv = cv, timeout = 500))
-nanotestaio(set_promise_context(cs, environment()))
+nanotestaio(cs <- request(req$context, "test", send_mode = "next", cv = cv, timeout = 500))
+if (later) nanotestaio(set_promise_context(cs, environment()))
 nanotestnn(cs$data)
 nanotest(typeof(ctxn <- .context(rep)) == "externalptr")
-nanotestaio(cr <- recv_aio_signal(ctxn, cv = cv, timeout = 500))
+nanotestaio(cr <- recv_aio(ctxn, cv = cv, timeout = 500))
 nanotest(call_aio(cr)$data == "test")
 nanotest(is.integer(send(ctxn, TRUE, mode = 0L, block = FALSE)))
 nanotest(typeof(ctxn <- .context(rep)) == "externalptr")
-nanotestaio(cs <- request(.context(req$socket), data = TRUE))
+nanotestaio(cs <- request(.context(req$socket), data = TRUE, cv = NA))
 nanotestnn(cs$data)
 nanotest(recv(ctxn, block = 500))
 nanotestz(send(ctxn, TRUE, mode = 3L, block = 500))
@@ -265,8 +268,8 @@ nanotestz(pipe_notify(rep, cv, add = TRUE, flag = TRUE))
 nanotestz(pipe_notify(rep, cv, remove = TRUE, flag = tools::SIGCONT))
 nanotestz(pipe_notify(req$socket, cv = cv, add = TRUE))
 nanotestz(pipe_notify(req$socket, cv = cv, cv2 = cv2, remove = TRUE, flag = tools::SIGCONT))
-nanotesterr(request_signal(err, "test", cv = cv), "valid")
-nanotesterr(recv_aio_signal(err, cv = cv, timeout = 500))
+nanotesterr(request(err, "test", cv = cv), "valid")
+nanotesterr(recv_aio(err, cv = cv, timeout = 500))
 nanotesterr(wait(err), "valid")
 nanotesterr(wait_(err), "valid")
 nanotesterr(until(err, 10), "valid")
@@ -424,7 +427,6 @@ nanotesterr(random("test"), "integer")
 nanotesterr(parse_url("tcp:/"), "argument")
 for (i in c(100:103, 200:208, 226, 300:308, 400:426, 428:431, 451, 500:511))
   nanotest(is.character(status_code(i)))
-nanotest(strcat("hello ", "world") == "hello world")
 
 s <- tryCatch(stream(dial = "wss://echo.websocket.events/", textframes = TRUE), error = function(e) NULL)
 if (length(s)) {
@@ -440,10 +442,10 @@ if (length(s)) {
   nanotest(length(call_aio(sr)[["data"]]))
   nanotestn(stop_aio(sr))
   nanotestaio(ss <- send_aio(s, "async", timeout = 500L))
-  nanotest(is.integer(call_aio(ss)[["result"]]))
+  nanotest(is.integer(ss[]))
   nanotestn(stop_aio(ss))
   nanotest(is.integer(send(s, 12.56, mode = "raw", block = 500L)))
-  nanotestaio(sr <- recv_aio_signal(s, mode = "double", timeout = 500L, cv = cv))
+  nanotestaio(sr <- recv_aio(s, mode = "double", timeout = 500L, cv = cv))
   nanotest(length(call_aio_(sr)[["data"]]))
   nanotest(cv_value(cv) > 0L)
   nanotest(is.character(opt(s, "ws:request-headers")))
@@ -530,16 +532,7 @@ nanotesterr(collect_aio_(list("a")), "object is not an Aio or list of Aios")
 nanotesterr(collect_aio(list(fakesock)), "object is not an Aio or list of Aios")
 nanotestn(stop_aio("a"))
 nanotestn(stop_aio(list("a")))
-nanotest(is.environment(set_promise_context(new.env(), new.env())))
-
-nanotest(base64enc("test") == "dGVzdA==")
-nanotest(base64dec(base64enc("test")) == "test")
-nanotest(is.character(base64enc(c("vector", "test"))))
-nanotest(is.raw(base64enc(data.frame(), convert = FALSE)))
-nanotest(is.raw(base64dec(base64enc(as.raw(c(1L, 2L)), convert = FALSE), convert = FALSE)))
-nanotest(is.integer(base64dec(base64enc(c(1L, 2L)), convert = NA)))
-nanotesterr(base64dec("__"), "not valid base64")
-nanotesterr(base64dec(404), "not valid base64")
+if (later) nanotest(is.environment(set_promise_context(new.env(), new.env())))
 
 pem <- "-----BEGIN CERTIFICATE----- -----END CERTIFICATE-----"
 test_tls <- function(pem) {
@@ -569,9 +562,17 @@ nanotestw(dial(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
 nanotestw(listen(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
 nanotestz(close(s1))
 nanotestz(close(s))
+if (requireNamespace("promises", quietly = TRUE)) {
+  nanotestaio(n <- ncurl_aio("https://postman-echo.com/get"))
+  nanotest(tryCatch(promises::is.promise(promises::then(n, cat)), error = function(e) TRUE))
+  nanotest(promises::is.promising(call_aio(n)))
+  nanotest(promises::is.promise(promises::as.promise(call_aio(ncurl_aio("https://postman-echo.com/get")))))
+  later::run_now()
+}
 if (Sys.info()[["sysname"]] == "Linux") {
   rm(list = ls())
   gc()
   Sys.sleep(1L)
   .Call(nanonext:::rnng_fini)
+  invisible()
 }
